@@ -63,6 +63,51 @@ static int test_query(yaz_sparql_t s, const char *pqf, const char *expect)
     return ret;
 }
 
+static int test_uri(yaz_sparql_t s, const char *uri, const char *schema,
+                    const char *expect)
+{
+    int ret = 0;
+    WRBUF addinfo = wrbuf_alloc();
+    WRBUF w = wrbuf_alloc();
+
+    int r = yaz_sparql_from_uri_wrbuf(s, addinfo, w, uri, schema);
+    if (expect)
+    {
+        if (!r)
+        {
+            if (!strcmp(expect, wrbuf_cstr(w)))
+                ret = 1;
+            else
+            {
+                yaz_log(YLOG_WARN, "test_sparql: uri=%s", uri);
+                yaz_log(YLOG_WARN, " expect: %s", expect);
+                yaz_log(YLOG_WARN, " got:    %s", wrbuf_cstr(w));
+            }
+        }
+        else
+        {
+            yaz_log(YLOG_WARN, "test_sparql: uri=%s", uri);
+            yaz_log(YLOG_WARN, " expect: %s", expect);
+            yaz_log(YLOG_WARN, " got error: %d:%s", r, wrbuf_cstr(addinfo));
+        }
+    }
+    else
+    {
+        if (r)
+            ret = 1;
+        else
+        {
+            yaz_log(YLOG_WARN, "test_sparql: uri=%s", uri);
+            yaz_log(YLOG_WARN, " expect error");
+            yaz_log(YLOG_WARN, " got:    %s", wrbuf_cstr(w));
+        }
+    }
+    wrbuf_destroy(w);
+    wrbuf_destroy(addinfo);
+    return ret;
+}
+
+
 static void tst1(void)
 {
     yaz_sparql_t s = yaz_sparql_create();
@@ -73,10 +118,7 @@ static void tst1(void)
                            "bf: <http://bibframe.org/vocab/>");
     yaz_sparql_add_pattern(s, "prefix",
                            "gs: http://gs.com/panorama/domain-model");
-    yaz_sparql_add_pattern(s, "field.title", "?title");
-    yaz_sparql_add_pattern(s, "field.author", "?author");
-    yaz_sparql_add_pattern(s, "field.description", "?description");
-    yaz_sparql_add_pattern(s, "field.instanceTitle", "?ititle");
+    yaz_sparql_add_pattern(s, "form", "SELECT ?title ?author ?description ?ititle");
     yaz_sparql_add_pattern(s, "criteria", "?work a bf:Work");
     yaz_sparql_add_pattern(s, "criteria", "?work bf:workTitle/bf:titleValue ?title");
     yaz_sparql_add_pattern(s, "criteria", "?work bf:creator/bf:label ?author");
@@ -107,12 +149,20 @@ static void tst1(void)
     yaz_sparql_add_pattern(s, "index.bf.targetAudience",
                            "?work bf:targetAudience %s");
     yaz_sparql_add_pattern(s, "index.bf.isbn", "?inst bf:ISBN %s");
+
+    yaz_sparql_add_pattern(s, "uri.full", "SELECT ?sub ?rel WHERE ?work = %u");
+
+    YAZ_CHECK(test_uri(s, "http://x/y", "full",
+                       "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns>\n"
+                       "PREFIX bf: <http://bibframe.org/vocab/>\n"
+                       "PREFIX gs: <http://gs.com/panorama/domain-model>\n"
+                       "SELECT ?sub ?rel WHERE ?work = <http://x/y>\n"));
+
     YAZ_CHECK(test_query(
                   s, "@attr 1=bf.title computer",
                   "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns>\n"
                   "PREFIX bf: <http://bibframe.org/vocab/>\n"
                   "PREFIX gs: <http://gs.com/panorama/domain-model>\n"
-                  "\n"
                   "SELECT ?title ?author ?description ?ititle\n"
                   "WHERE {\n"
                   "  ?work a bf:Work .\n"
@@ -132,7 +182,6 @@ static void tst1(void)
                   "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns>\n"
                   "PREFIX bf: <http://bibframe.org/vocab/>\n"
                   "PREFIX gs: <http://gs.com/panorama/domain-model>\n"
-                  "\n"
                   "SELECT ?title ?author ?description ?ititle\n"
                   "WHERE {\n"
                   "  ?work a bf:Work .\n"
@@ -152,7 +201,6 @@ static void tst1(void)
                   "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns>\n"
                   "PREFIX bf: <http://bibframe.org/vocab/>\n"
                   "PREFIX gs: <http://gs.com/panorama/domain-model>\n"
-                  "\n"
                   "SELECT ?title ?author ?description ?ititle\n"
                   "WHERE {\n"
                   "  ?work a bf:Work .\n"
@@ -173,7 +221,6 @@ static void tst1(void)
                   "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns>\n"
                   "PREFIX bf: <http://bibframe.org/vocab/>\n"
                   "PREFIX gs: <http://gs.com/panorama/domain-model>\n"
-                  "\n"
                   "SELECT ?title ?author ?description ?ititle\n"
                   "WHERE {\n"
                   "  ?work a bf:Work .\n"
@@ -198,7 +245,6 @@ static void tst1(void)
                   "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns>\n"
                   "PREFIX bf: <http://bibframe.org/vocab/>\n"
                   "PREFIX gs: <http://gs.com/panorama/domain-model>\n"
-                  "\n"
                   "SELECT ?title ?author ?description ?ititle\n"
                   "WHERE {\n"
                   "  ?work a bf:Work .\n"
@@ -228,7 +274,6 @@ static void tst1(void)
                   "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns>\n"
                   "PREFIX bf: <http://bibframe.org/vocab/>\n"
                   "PREFIX gs: <http://gs.com/panorama/domain-model>\n"
-                  "\n"
                   "SELECT ?title ?author ?description ?ititle\n"
                   "WHERE {\n"
                   "  ?work a bf:Work .\n"
@@ -255,7 +300,6 @@ static void tst1(void)
                   "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns>\n"
                   "PREFIX bf: <http://bibframe.org/vocab/>\n"
                   "PREFIX gs: <http://gs.com/panorama/domain-model>\n"
-                  "\n"
                   "SELECT ?title ?author ?description ?ititle\n"
                   "WHERE {\n"
                   "  ?work a bf:Work .\n"
@@ -280,7 +324,6 @@ static void tst1(void)
                   "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns>\n"
                   "PREFIX bf: <http://bibframe.org/vocab/>\n"
                   "PREFIX gs: <http://gs.com/panorama/domain-model>\n"
-                  "\n"
                   "SELECT ?title ?author ?description ?ititle\n"
                   "WHERE {\n"
                   "  ?work a bf:Work .\n"
@@ -301,7 +344,6 @@ static void tst1(void)
                   "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns>\n"
                   "PREFIX bf: <http://bibframe.org/vocab/>\n"
                   "PREFIX gs: <http://gs.com/panorama/domain-model>\n"
-                  "\n"
                   "SELECT ?title ?author ?description ?ititle\n"
                   "WHERE {\n"
                   "  ?work a bf:Work .\n"
@@ -329,10 +371,7 @@ static void tst2(void)
                            "bf: <http://bibframe.org/vocab/>");
     yaz_sparql_add_pattern(s, "prefix",
                            "gs: http://gs.com/panorama/domain-model");
-    yaz_sparql_add_pattern(s, "field.title", "?title");
-    yaz_sparql_add_pattern(s, "field.author", "?author");
-    yaz_sparql_add_pattern(s, "field.description", "?description");
-    yaz_sparql_add_pattern(s, "field.instanceTitle", "?ititle");
+    yaz_sparql_add_pattern(s, "form", "SELECT ?title ?author ?description ?ititle");
     yaz_sparql_add_pattern(s, "criteria", "?work a bf:Work");
     yaz_sparql_add_pattern(s, "criteria", "?work bf:workTitle/bf:titleValue ?title");
     yaz_sparql_add_pattern(s, "criteria", "?work bf:creator/bf:label ?author");
@@ -368,7 +407,6 @@ static void tst2(void)
                   "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns>\n"
                   "PREFIX bf: <http://bibframe.org/vocab/>\n"
                   "PREFIX gs: <http://gs.com/panorama/domain-model>\n"
-                  "\n"
                   "SELECT ?title ?author ?description ?ititle\n"
                   "WHERE {\n"
                   "  ?work a bf:Work .\n"
@@ -388,7 +426,6 @@ static void tst2(void)
                   "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns>\n"
                   "PREFIX bf: <http://bibframe.org/vocab/>\n"
                   "PREFIX gs: <http://gs.com/panorama/domain-model>\n"
-                  "\n"
                   "SELECT ?title ?author ?description ?ititle\n"
                   "WHERE {\n"
                   "  ?work a bf:Work .\n"
@@ -407,7 +444,6 @@ static void tst2(void)
                   "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns>\n"
                   "PREFIX bf: <http://bibframe.org/vocab/>\n"
                   "PREFIX gs: <http://gs.com/panorama/domain-model>\n"
-                  "\n"
                   "SELECT ?title ?author ?description ?ititle\n"
                   "WHERE {\n"
                   "  ?work a bf:Work .\n"
